//
// Generated by JTB 1.3.2
//
package org.ow2.mind.adl.jtb.visitor;

import org.ow2.mind.adl.jtb.syntaxtree.*;
import java.util.*;

/**
 * A skeleton output formatter for your language grammar.  Using the
 * add() method along with force(), indent(), and outdent(), you can
 * easily specify how this visitor will format the given syntax tree.
 * See the JTB documentation for more details.
 *
 * Pass your syntax tree to this visitor, and then to the TreeDumper
 * visitor in order to "pretty print" your tree.
 */
public class TreeFormatter extends DepthFirstVisitor {
   private Vector<FormatCommand> cmdQueue = new Vector<FormatCommand>();
   private boolean lineWrap;
   private int wrapWidth;
   private int indentAmt;
   private int curLine = 1;
   private int curColumn = 1;
   private int curIndent = 0;

   /**
    * The default constructor assumes an indentation amount of 3 spaces
    * and no line-wrap.  You may alternately use the other constructor to
    * specify your own indentation amount and line width.
    */
   public TreeFormatter() { this(3, 0); }

   /**
    * This constructor accepts an indent amount and a line width which is
    * used to wrap long lines.  If a token's beginColumn value is greater
    * than the specified wrapWidth, it will be moved to the next line and
    * indented one extra level.  To turn off line-wrapping, specify a
    * wrapWidth of 0.
    *
    * @param   indentAmt   Amount of spaces per indentation level.
    * @param   wrapWidth   Wrap lines longer than wrapWidth.  0 for no wrap.
    */
   public TreeFormatter(int indentAmt, int wrapWidth) {
      this.indentAmt = indentAmt;
      this.wrapWidth = wrapWidth;

      if ( wrapWidth > 0 )
         lineWrap = true;
      else
         lineWrap = false;
   }

   /**
    * Accepts a NodeListInterface object and performs an optional format
    * command between each node in the list (but not after the last node).
    */
   protected void processList(NodeListInterface n) {
      processList(n, null);
   }

   protected void processList(NodeListInterface n, FormatCommand cmd) {
      for ( Enumeration<Node> e = n.elements(); e.hasMoreElements(); ) {
         e.nextElement().accept(this);
         if ( cmd != null && e.hasMoreElements() )
            cmdQueue.addElement(cmd);
      }
   }

   /**
    * A Force command inserts a line break and indents the next line to
    * the current indentation level.  Use "add(force());".
    */
   protected FormatCommand force() { return force(1); }
   protected FormatCommand force(int i) {
      return new FormatCommand(FormatCommand.FORCE, i);
   }

   /**
    * An Indent command increases the indentation level by one (or a
    * user-specified amount).  Use "add(indent());".
    */
   protected FormatCommand indent() { return indent(1); }
   protected FormatCommand indent(int i) {
      return new FormatCommand(FormatCommand.INDENT, i);
   }

   /**
    * An Outdent command is the reverse of the Indent command: it reduces
    * the indentation level.  Use "add(outdent());".
    */
   protected FormatCommand outdent() { return outdent(1); }
   protected FormatCommand outdent(int i) {
      return new FormatCommand(FormatCommand.OUTDENT, i);
   }

   /**
    * A Space command simply adds one or a user-specified number of
    * spaces between tokens.  Use "add(space());".
    */
   protected FormatCommand space() { return space(1); }
   protected FormatCommand space(int i) {
      return new FormatCommand(FormatCommand.SPACE, i);
   }

   /**
    * Use this method to add FormatCommands to the command queue to be
    * executed when the next token in the tree is visited.
    */
   protected void add(FormatCommand cmd) {
      cmdQueue.addElement(cmd);
   }

   /**
    * Executes the commands waiting in the command queue, then inserts the
    * proper location information into the current NodeToken.
    *
    * If there are any special tokens preceding this token, they will be
    * given the current location information.  The token will follow on
    * the next line, at the proper indentation level.  If this is not the
    * behavior you want from special tokens, feel free to modify this
    * method.
    */
   public void visit(NodeToken n) {
      for ( Enumeration<FormatCommand> e = cmdQueue.elements(); e.hasMoreElements(); ) {
         FormatCommand cmd = e.nextElement();
         switch ( cmd.getCommand() ) {
         case FormatCommand.FORCE :
            curLine += cmd.getNumCommands();
            curColumn = curIndent + 1;
            break;
         case FormatCommand.INDENT :
            curIndent += indentAmt * cmd.getNumCommands();
            break;
         case FormatCommand.OUTDENT :
            if ( curIndent >= indentAmt )
               curIndent -= indentAmt * cmd.getNumCommands();
            break;
         case FormatCommand.SPACE :
            curColumn += cmd.getNumCommands();
            break;
         default :
            throw new TreeFormatterException(
               "Invalid value in command queue.");
         }
      }

      cmdQueue.removeAllElements();

      //
      // Handle all special tokens preceding this NodeToken
      //
      if ( n.numSpecials() > 0 )
         for ( Enumeration<NodeToken> e = n.specialTokens.elements();
               e.hasMoreElements(); ) {
            NodeToken special = e.nextElement();

            //
            // -Place the token.
            // -Move cursor to next line after the special token.
            // -Don't update curColumn--want to keep current indent level.
            //
            placeToken(special, curLine, curColumn);
            curLine = special.endLine + 1;
         }

      placeToken(n, curLine, curColumn);
      curLine = n.endLine;
      curColumn = n.endColumn;
   }

   /**
    * Inserts token location (beginLine, beginColumn, endLine, endColumn)
    * information into the NodeToken.  Takes into account line-wrap.
    * Does not update curLine and curColumn.
    */
   private void placeToken(NodeToken n, int line, int column) {
      int length = n.tokenImage.length();

      //
      // Find beginning of token.  Only line-wrap for single-line tokens
      //
      if ( !lineWrap || n.tokenImage.indexOf('\n') != -1 ||
           column + length <= wrapWidth )
         n.beginColumn = column;
      else {
         ++line;
         column = curIndent + indentAmt + 1;
         n.beginColumn = column;
      }

      n.beginLine = line;

      //
      // Find end of token; don't count \n if it's the last character
      //
      for ( int i = 0; i < length; ++i ) {
         if ( n.tokenImage.charAt(i) == '\n' && i < length - 1 ) {
            ++line;
            column = 1;
         }
         else
            ++column;
      }

      n.endLine = line;
      n.endColumn = column;
   }

   //
   // User-generated visitor methods below
   //

   /**
    * <PRE>
    * f0 -> ( ImportDefinition() )*
    * f1 -> ArchitectureDefinition()
    * f2 -> &lt;EOF&gt;
    * </PRE>
    */
   public void visit(ADLFile n) {
      if ( n.f0.present() ) {
         processList(n.f0);
      }
      n.f1.accept(this);
      n.f2.accept(this);
   }

   /**
    * <PRE>
    * f0 -> ( Annotation() )*
    * f1 -> &lt;IMPORT&gt;
    * f2 -> &lt;IDENTIFIER&gt;
    * f3 -> &lt;DOT&gt;
    * f4 -> ( &lt;IDENTIFIER&gt; &lt;DOT&gt; )*
    * f5 -> ( &lt;START&gt; | &lt;IDENTIFIER&gt; )
    * f6 -> [ &lt;SEMICOLON&gt; ]
    * </PRE>
    */
   public void visit(ImportDefinition n) {
      if ( n.f0.present() ) {
         processList(n.f0);
      }
      n.f1.accept(this);
      n.f2.accept(this);
      n.f3.accept(this);
      if ( n.f4.present() ) {
         processList(n.f4);
      }
      n.f5.accept(this);
      if ( n.f6.present() ) {
         n.f6.accept(this);
      }
   }

   /**
    * <PRE>
    * f0 -> ( TypeDefinition() | PrimitiveDefinition() | CompositeDefinition() )
    * </PRE>
    */
   public void visit(ArchitectureDefinition n) {
      n.f0.accept(this);
   }

   /**
    * <PRE>
    * f0 -> Annotations()
    * f1 -> &lt;TYPE&gt;
    * f2 -> FullyQualifiedName()
    * f3 -> [ ExtendedTypeDefinitions() ]
    * f4 -> TypeDefinitionBody()
    * </PRE>
    */
   public void visit(TypeDefinition n) {
      n.f0.accept(this);
      n.f1.accept(this);
      n.f2.accept(this);
      if ( n.f3.present() ) {
         n.f3.accept(this);
      }
      n.f4.accept(this);
   }

   /**
    * <PRE>
    * f0 -> &lt;EXTENDS&gt;
    * f1 -> TypeDefinitionReference()
    * f2 -> ( &lt;COMMA&gt; TypeDefinitionReference() )*
    * </PRE>
    */
   public void visit(ExtendedTypeDefinitions n) {
      n.f0.accept(this);
      n.f1.accept(this);
      if ( n.f2.present() ) {
         processList(n.f2);
      }
   }

   /**
    * <PRE>
    * f0 -> FullyQualifiedName()
    * </PRE>
    */
   public void visit(TypeDefinitionReference n) {
      n.f0.accept(this);
   }

   /**
    * <PRE>
    * f0 -> &lt;LBRACE&gt; ( TypeDefinitionElement() )* &lt;RBRACE&gt;
    *       | ( TypeDefinitionElement() )*
    * </PRE>
    */
   public void visit(TypeDefinitionBody n) {
      n.f0.accept(this);
   }

   /**
    * <PRE>
    * f0 -> InterfaceDefinition()
    * </PRE>
    */
   public void visit(TypeDefinitionElement n) {
      n.f0.accept(this);
   }

   /**
    * <PRE>
    * f0 -> Annotations()
    * f1 -> [ &lt;ABSTRACT&gt; ]
    * f2 -> &lt;PRIMITIVE&gt;
    * f3 -> FullyQualifiedName()
    * f4 -> [ FormalParameterDeclarationList() ]
    * f5 -> [ ExtendedPrimitiveDefinitions() ]
    * f6 -> PrimitiveDefinitionBody()
    * </PRE>
    */
   public void visit(PrimitiveDefinition n) {
      n.f0.accept(this);
      if ( n.f1.present() ) {
         n.f1.accept(this);
      }
      n.f2.accept(this);
      n.f3.accept(this);
      if ( n.f4.present() ) {
         n.f4.accept(this);
      }
      if ( n.f5.present() ) {
         n.f5.accept(this);
      }
      n.f6.accept(this);
   }

   /**
    * <PRE>
    * f0 -> &lt;LPAREN&gt;
    * f1 -> [ FormalParameterDeclaration() ( &lt;COMMA&gt; FormalParameterDeclaration() )* ]
    * f2 -> &lt;RPAREN&gt;
    * </PRE>
    */
   public void visit(FormalParameterDeclarationList n) {
      n.f0.accept(this);
      if ( n.f1.present() ) {
         n.f1.accept(this);
      }
      n.f2.accept(this);
   }

   /**
    * <PRE>
    * f0 -> &lt;IDENTIFIER&gt;
    * </PRE>
    */
   public void visit(FormalParameterDeclaration n) {
      n.f0.accept(this);
   }

   /**
    * <PRE>
    * f0 -> &lt;EXTENDS&gt;
    * f1 -> PrimitiveDefinitionReference()
    * f2 -> ( &lt;COMMA&gt; PrimitiveDefinitionReference() )*
    * </PRE>
    */
   public void visit(ExtendedPrimitiveDefinitions n) {
      n.f0.accept(this);
      n.f1.accept(this);
      if ( n.f2.present() ) {
         processList(n.f2);
      }
   }

   /**
    * <PRE>
    * f0 -> FullyQualifiedName()
    * f1 -> [ ArgumentList() ]
    * </PRE>
    */
   public void visit(PrimitiveDefinitionReference n) {
      n.f0.accept(this);
      if ( n.f1.present() ) {
         n.f1.accept(this);
      }
   }

   /**
    * <PRE>
    * f0 -> &lt;LPAREN&gt;
    * f1 -> [ ArgumentAssignement() ( &lt;COMMA&gt; ArgumentAssignement() )* | Argument() ( &lt;COMMA&gt; Argument() )* ]
    * f2 -> &lt;RPAREN&gt;
    * </PRE>
    */
   public void visit(ArgumentList n) {
      n.f0.accept(this);
      if ( n.f1.present() ) {
         n.f1.accept(this);
      }
      n.f2.accept(this);
   }

   /**
    * <PRE>
    * f0 -> &lt;IDENTIFIER&gt;
    * f1 -> &lt;ASSIGN&gt;
    * f2 -> Argument()
    * </PRE>
    */
   public void visit(ArgumentAssignement n) {
      n.f0.accept(this);
      n.f1.accept(this);
      n.f2.accept(this);
   }

   /**
    * <PRE>
    * f0 -> StringValue()
    *       | IntegerValue()
    *       | ReferenceValue()
    *       | NullValue()
    * </PRE>
    */
   public void visit(Argument n) {
      n.f0.accept(this);
   }

   /**
    * <PRE>
    * f0 -> &lt;LBRACE&gt; ( PrimitiveDefinitionElement() )* &lt;RBRACE&gt;
    *       | ( PrimitiveDefinitionElement() )*
    * </PRE>
    */
   public void visit(PrimitiveDefinitionBody n) {
      n.f0.accept(this);
   }

   /**
    * <PRE>
    * f0 -> InterfaceDefinition()
    *       | AttributeDefinition()
    *       | DataDefinition()
    *       | ImplementationDefinition()
    * </PRE>
    */
   public void visit(PrimitiveDefinitionElement n) {
      n.f0.accept(this);
   }

   /**
    * <PRE>
    * f0 -> Annotations()
    * f1 -> &lt;COMPOSITE&gt;
    * f2 -> FullyQualifiedName()
    * f3 -> [ FormalTypeParameterDeclarationList() ]
    * f4 -> [ FormalParameterDeclarationList() ]
    * f5 -> [ ExtendedCompositeDefinitions() ]
    * f6 -> CompositeDefinitionBody()
    * </PRE>
    */
   public void visit(CompositeDefinition n) {
      n.f0.accept(this);
      n.f1.accept(this);
      n.f2.accept(this);
      if ( n.f3.present() ) {
         n.f3.accept(this);
      }
      if ( n.f4.present() ) {
         n.f4.accept(this);
      }
      if ( n.f5.present() ) {
         n.f5.accept(this);
      }
      n.f6.accept(this);
   }

   /**
    * <PRE>
    * f0 -> &lt;LT&gt;
    * f1 -> FormalTypeParameterDeclaration()
    * f2 -> ( &lt;COMMA&gt; FormalTypeParameterDeclaration() )*
    * f3 -> &lt;GT&gt;
    * </PRE>
    */
   public void visit(FormalTypeParameterDeclarationList n) {
      n.f0.accept(this);
      n.f1.accept(this);
      if ( n.f2.present() ) {
         processList(n.f2);
      }
      n.f3.accept(this);
   }

   /**
    * <PRE>
    * f0 -> &lt;IDENTIFIER&gt;
    * f1 -> &lt;CONFORMSTO&gt;
    * f2 -> TypeDefinitionReference()
    * </PRE>
    */
   public void visit(FormalTypeParameterDeclaration n) {
      n.f0.accept(this);
      n.f1.accept(this);
      n.f2.accept(this);
   }

   /**
    * <PRE>
    * f0 -> FullyQualifiedName()
    * f1 -> [ TypeArgumentList() ]
    * f2 -> [ ArgumentList() ]
    * </PRE>
    */
   public void visit(CompositeDefinitionReference n) {
      n.f0.accept(this);
      if ( n.f1.present() ) {
         n.f1.accept(this);
      }
      if ( n.f2.present() ) {
         n.f2.accept(this);
      }
   }

   /**
    * <PRE>
    * f0 -> &lt;LT&gt;
    * f1 -> [ TypeArgument() ( &lt;COMMA&gt; TypeArgument() )* | TypeArgumentAssignement() ( &lt;COMMA&gt; TypeArgumentAssignement() )* ]
    * f2 -> &lt;GT&gt;
    * </PRE>
    */
   public void visit(TypeArgumentList n) {
      n.f0.accept(this);
      if ( n.f1.present() ) {
         n.f1.accept(this);
      }
      n.f2.accept(this);
   }

   /**
    * <PRE>
    * f0 -> &lt;IDENTIFIER&gt;
    * f1 -> &lt;ASSIGN&gt;
    * f2 -> TypeArgument()
    * </PRE>
    */
   public void visit(TypeArgumentAssignement n) {
      n.f0.accept(this);
      n.f1.accept(this);
      n.f2.accept(this);
   }

   /**
    * <PRE>
    * f0 -> CompositeDefinitionReference()
    *       | &lt;ANY&gt;
    * </PRE>
    */
   public void visit(TypeArgument n) {
      n.f0.accept(this);
   }

   /**
    * <PRE>
    * f0 -> &lt;EXTENDS&gt;
    * f1 -> CompositeDefinitionReference()
    * f2 -> ( &lt;COMMA&gt; CompositeDefinitionReference() )*
    * </PRE>
    */
   public void visit(ExtendedCompositeDefinitions n) {
      n.f0.accept(this);
      n.f1.accept(this);
      if ( n.f2.present() ) {
         processList(n.f2);
      }
   }

   /**
    * <PRE>
    * f0 -> &lt;LBRACE&gt; ( CompositeDefinitionElement() )* &lt;RBRACE&gt;
    *       | ( CompositeDefinitionElement() )*
    * </PRE>
    */
   public void visit(CompositeDefinitionBody n) {
      n.f0.accept(this);
   }

   /**
    * <PRE>
    * f0 -> InterfaceDefinition()
    *       | BindingDefinition()
    *       | SubComponentDefinition()
    * </PRE>
    */
   public void visit(CompositeDefinitionElement n) {
      n.f0.accept(this);
   }

   /**
    * <PRE>
    * f0 -> Annotations()
    * f1 -> ( &lt;PROVIDES&gt; | &lt;REQUIRES&gt; )
    * f2 -> [ &lt;OPTIONAL&gt; ]
    * f3 -> FullyQualifiedName()
    * f4 -> &lt;AS&gt;
    * f5 -> &lt;IDENTIFIER&gt;
    * f6 -> [ &lt;LBRACKET&gt; [ &lt;INTEGER_LITERAL&gt; ] &lt;RBRACKET&gt; ]
    * f7 -> [ &lt;SEMICOLON&gt; ]
    * </PRE>
    */
   public void visit(InterfaceDefinition n) {
      n.f0.accept(this);
      n.f1.accept(this);
      if ( n.f2.present() ) {
         n.f2.accept(this);
      }
      n.f3.accept(this);
      n.f4.accept(this);
      n.f5.accept(this);
      if ( n.f6.present() ) {
         n.f6.accept(this);
      }
      if ( n.f7.present() ) {
         n.f7.accept(this);
      }
   }

   /**
    * <PRE>
    * f0 -> Annotations()
    * f1 -> &lt;ATTRIBUTE&gt;
    * f2 -> [ AttributeType() ]
    * f3 -> &lt;IDENTIFIER&gt;
    * f4 -> [ &lt;ASSIGN&gt; AttributeValue() ]
    * f5 -> [ &lt;SEMICOLON&gt; ]
    * </PRE>
    */
   public void visit(AttributeDefinition n) {
      n.f0.accept(this);
      n.f1.accept(this);
      if ( n.f2.present() ) {
         n.f2.accept(this);
      }
      n.f3.accept(this);
      if ( n.f4.present() ) {
         n.f4.accept(this);
      }
      if ( n.f5.present() ) {
         n.f5.accept(this);
      }
   }

   /**
    * <PRE>
    * f0 -> &lt;INT&gt;
    *       | &lt;STRING&gt;
    *       | &lt;INT8_T&gt;
    *       | &lt;UINT8_T&gt;
    *       | &lt;INT16_T&gt;
    *       | &lt;UINT16_T&gt;
    *       | &lt;INT32_T&gt;
    *       | &lt;UINT32_T&gt;
    *       | &lt;INT64_T&gt;
    *       | &lt;UINT64_T&gt;
    *       | &lt;INTPTR_T&gt;
    *       | &lt;UINTPTR_T&gt;
    * </PRE>
    */
   public void visit(AttributeType n) {
      n.f0.accept(this);
   }

   /**
    * <PRE>
    * f0 -> StringValue()
    *       | IntegerValue()
    *       | ReferenceValue()
    *       | NullValue()
    * </PRE>
    */
   public void visit(AttributeValue n) {
      n.f0.accept(this);
   }

   /**
    * <PRE>
    * f0 -> Annotations()
    * f1 -> &lt;DATA&gt;
    * f2 -> ( Path() | &lt;INLINED_CODE&gt; )
    * f3 -> [ &lt;SEMICOLON&gt; ]
    * </PRE>
    */
   public void visit(DataDefinition n) {
      n.f0.accept(this);
      n.f1.accept(this);
      n.f2.accept(this);
      if ( n.f3.present() ) {
         n.f3.accept(this);
      }
   }

   /**
    * <PRE>
    * f0 -> Annotations()
    * f1 -> &lt;SOURCE&gt;
    * f2 -> ( Path() | &lt;INLINED_CODE&gt; )
    * f3 -> [ &lt;SEMICOLON&gt; ]
    * </PRE>
    */
   public void visit(ImplementationDefinition n) {
      n.f0.accept(this);
      n.f1.accept(this);
      n.f2.accept(this);
      if ( n.f3.present() ) {
         n.f3.accept(this);
      }
   }

   /**
    * <PRE>
    * f0 -> Annotations()
    * f1 -> &lt;BINDS&gt;
    * f2 -> BindingComponentName()
    * f3 -> &lt;DOT&gt;
    * f4 -> &lt;IDENTIFIER&gt;
    * f5 -> [ &lt;LBRACKET&gt; &lt;INTEGER_LITERAL&gt; &lt;RBRACKET&gt; ]
    * f6 -> &lt;TO&gt;
    * f7 -> BindingComponentName()
    * f8 -> &lt;DOT&gt;
    * f9 -> &lt;IDENTIFIER&gt;
    * f10 -> [ &lt;LBRACKET&gt; &lt;INTEGER_LITERAL&gt; &lt;RBRACKET&gt; ]
    * f11 -> [ &lt;SEMICOLON&gt; ]
    * </PRE>
    */
   public void visit(BindingDefinition n) {
      n.f0.accept(this);
      n.f1.accept(this);
      n.f2.accept(this);
      n.f3.accept(this);
      n.f4.accept(this);
      if ( n.f5.present() ) {
         n.f5.accept(this);
      }
      n.f6.accept(this);
      n.f7.accept(this);
      n.f8.accept(this);
      n.f9.accept(this);
      if ( n.f10.present() ) {
         n.f10.accept(this);
      }
      if ( n.f11.present() ) {
         n.f11.accept(this);
      }
   }

   /**
    * <PRE>
    * f0 -> &lt;IDENTIFIER&gt;
    *       | &lt;THIS&gt;
    * </PRE>
    */
   public void visit(BindingComponentName n) {
      n.f0.accept(this);
   }

   /**
    * <PRE>
    * f0 -> Annotations()
    * f1 -> &lt;CONTAINS&gt;
    * f2 -> SimpleSubComponentReference()
    * f3 -> [ &lt;SEMICOLON&gt; ]
    * </PRE>
    */
   public void visit(SubComponentDefinition n) {
      n.f0.accept(this);
      n.f1.accept(this);
      n.f2.accept(this);
      if ( n.f3.present() ) {
         n.f3.accept(this);
      }
   }

   /**
    * <PRE>
    * f0 -> [ CompositeDefinitionReference() ]
    * f1 -> &lt;AS&gt;
    * f2 -> &lt;IDENTIFIER&gt;
    * f3 -> [ ( PrimitiveAnonymousDefinition() | CompositeAnonymousDefinition() ) ]
    * </PRE>
    */
   public void visit(SimpleSubComponentReference n) {
      if ( n.f0.present() ) {
         n.f0.accept(this);
      }
      n.f1.accept(this);
      n.f2.accept(this);
      if ( n.f3.present() ) {
         n.f3.accept(this);
      }
   }

   /**
    * <PRE>
    * f0 -> Annotations()
    * f1 -> &lt;PRIMITIVE&gt;
    * f2 -> &lt;LBRACE&gt;
    * f3 -> ( PrimitiveDefinitionElement() )*
    * f4 -> &lt;RBRACE&gt;
    * </PRE>
    */
   public void visit(PrimitiveAnonymousDefinition n) {
      n.f0.accept(this);
      n.f1.accept(this);
      n.f2.accept(this);
      if ( n.f3.present() ) {
         processList(n.f3);
      }
      n.f4.accept(this);
   }

   /**
    * <PRE>
    * f0 -> Annotations()
    * f1 -> &lt;COMPOSITE&gt;
    * f2 -> &lt;LBRACE&gt;
    * f3 -> ( CompositeDefinitionElement() )*
    * f4 -> &lt;RBRACE&gt;
    * </PRE>
    */
   public void visit(CompositeAnonymousDefinition n) {
      n.f0.accept(this);
      n.f1.accept(this);
      n.f2.accept(this);
      if ( n.f3.present() ) {
         processList(n.f3);
      }
      n.f4.accept(this);
   }

   /**
    * <PRE>
    * f0 -> Annotations()
    * f1 -> &lt;LBRACE&gt;
    * f2 -> ( PrimitiveDefinitionElement() )*
    * f3 -> &lt;RBRACE&gt;
    * </PRE>
    */
   public void visit(PrimitiveAnonymousExtension n) {
      n.f0.accept(this);
      n.f1.accept(this);
      if ( n.f2.present() ) {
         processList(n.f2);
      }
      n.f3.accept(this);
   }

   /**
    * <PRE>
    * f0 -> Annotations()
    * f1 -> &lt;LBRACE&gt;
    * f2 -> ( CompositeDefinitionElement() )*
    * f3 -> &lt;RBRACE&gt;
    * </PRE>
    */
   public void visit(CompositeAnonymousExtension n) {
      n.f0.accept(this);
      n.f1.accept(this);
      if ( n.f2.present() ) {
         processList(n.f2);
      }
      n.f3.accept(this);
   }

   /**
    * <PRE>
    * f0 -> ( Annotation() )*
    * </PRE>
    */
   public void visit(Annotations n) {
      if ( n.f0.present() ) {
         processList(n.f0);
      }
   }

   /**
    * <PRE>
    * f0 -> &lt;AT&gt;
    * f1 -> FullyQualifiedName()
    * f2 -> [ AnnotationParameters() ]
    * </PRE>
    */
   public void visit(Annotation n) {
      n.f0.accept(this);
      n.f1.accept(this);
      if ( n.f2.present() ) {
         n.f2.accept(this);
      }
   }

   /**
    * <PRE>
    * f0 -> &lt;LPAREN&gt;
    * f1 -> [ AnnotationValuePairs() | AnnotationValue() ]
    * f2 -> &lt;RPAREN&gt;
    * </PRE>
    */
   public void visit(AnnotationParameters n) {
      n.f0.accept(this);
      if ( n.f1.present() ) {
         n.f1.accept(this);
      }
      n.f2.accept(this);
   }

   /**
    * <PRE>
    * f0 -> AnnotationValuePair()
    * f1 -> ( &lt;COMMA&gt; AnnotationValuePair() )*
    * </PRE>
    */
   public void visit(AnnotationValuePairs n) {
      n.f0.accept(this);
      if ( n.f1.present() ) {
         processList(n.f1);
      }
   }

   /**
    * <PRE>
    * f0 -> &lt;IDENTIFIER&gt;
    * f1 -> &lt;ASSIGN&gt;
    * f2 -> AnnotationValue()
    * </PRE>
    */
   public void visit(AnnotationValuePair n) {
      n.f0.accept(this);
      n.f1.accept(this);
      n.f2.accept(this);
   }

   /**
    * <PRE>
    * f0 -> StringValue()
    *       | IntegerValue()
    *       | BooleanValue()
    *       | AnnotationAnnotationValue()
    *       | ArrayAnnotationValue()
    *       | NullValue()
    * </PRE>
    */
   public void visit(AnnotationValue n) {
      n.f0.accept(this);
   }

   /**
    * <PRE>
    * f0 -> Annotation()
    * </PRE>
    */
   public void visit(AnnotationAnnotationValue n) {
      n.f0.accept(this);
   }

   /**
    * <PRE>
    * f0 -> &lt;LBRACE&gt;
    * f1 -> [ AnnotationValue() ( &lt;COMMA&gt; AnnotationValue() )* ]
    * f2 -> &lt;RBRACE&gt;
    * </PRE>
    */
   public void visit(ArrayAnnotationValue n) {
      n.f0.accept(this);
      if ( n.f1.present() ) {
         n.f1.accept(this);
      }
      n.f2.accept(this);
   }

   /**
    * <PRE>
    * f0 -> &lt;STRING_LITERAL&gt;
    * </PRE>
    */
   public void visit(StringValue n) {
      n.f0.accept(this);
   }

   /**
    * <PRE>
    * f0 -> [ &lt;MINUS&gt; | &lt;PLUS&gt; ]
    * f1 -> &lt;INTEGER_LITERAL&gt;
    * </PRE>
    */
   public void visit(IntegerValue n) {
      if ( n.f0.present() ) {
         n.f0.accept(this);
      }
      n.f1.accept(this);
   }

   /**
    * <PRE>
    * f0 -> &lt;BOOLEAN_LITERAL&gt;
    * </PRE>
    */
   public void visit(BooleanValue n) {
      n.f0.accept(this);
   }

   /**
    * <PRE>
    * f0 -> &lt;IDENTIFIER&gt;
    * </PRE>
    */
   public void visit(ReferenceValue n) {
      n.f0.accept(this);
   }

   /**
    * <PRE>
    * f0 -> &lt;NULL&gt;
    * </PRE>
    */
   public void visit(NullValue n) {
      n.f0.accept(this);
   }

   /**
    * <PRE>
    * f0 -> &lt;IDENTIFIER&gt;
    * f1 -> ( &lt;DOT&gt; &lt;IDENTIFIER&gt; )*
    * </PRE>
    */
   public void visit(FullyQualifiedName n) {
      n.f0.accept(this);
      if ( n.f1.present() ) {
         processList(n.f1);
      }
   }

   /**
    * <PRE>
    * f0 -> [ &lt;SLASH&gt; ]
    * f1 -> [ &lt;DOT&gt; &lt;SLASH&gt; ]
    * f2 -> ( &lt;DOTDOT&gt; &lt;SLASH&gt; )*
    * f3 -> &lt;IDENTIFIER&gt;
    * f4 -> ( &lt;SLASH&gt; &lt;IDENTIFIER&gt; )*
    * f5 -> &lt;DOT&gt;
    * f6 -> &lt;IDENTIFIER&gt;
    * </PRE>
    */
   public void visit(Path n) {
      if ( n.f0.present() ) {
         n.f0.accept(this);
      }
      if ( n.f1.present() ) {
         n.f1.accept(this);
      }
      if ( n.f2.present() ) {
         processList(n.f2);
      }
      n.f3.accept(this);
      if ( n.f4.present() ) {
         processList(n.f4);
      }
      n.f5.accept(this);
      n.f6.accept(this);
   }

}

class FormatCommand {
   public static final int FORCE = 0;
   public static final int INDENT = 1;
   public static final int OUTDENT = 2;
   public static final int SPACE = 3;

   private int command;
   private int numCommands;

   FormatCommand(int command, int numCommands) {
      this.command = command;
      this.numCommands = numCommands;
   }

   public int getCommand()             { return command; }
   public int getNumCommands()         { return numCommands; }
   public void setCommand(int i)       { command = i; }
   public void setNumCommands(int i)   { numCommands = i; }
}

class TreeFormatterException extends RuntimeException {
   TreeFormatterException()         { super(); }
   TreeFormatterException(String s) { super(s); }
}
